package com.jiuqi.util;

import java.util.ArrayList;

public final class IntMatrix
{
    private int colCount;
    private int rowCount;
    private ArrayList rowList;
    
    public IntMatrix() {
        this.colCount = 0;
        this.rowCount = 0;
        this.rowList = new ArrayList(0);
    }
    
    public IntMatrix(final int colCount, final int rowCount) {
        this.colCount = 0;
        this.rowCount = 0;
        this.rowList = new ArrayList(0);
        this.reset(colCount, rowCount);
    }
    
    public void clear() {
        this.rowList.clear();
        this.colCount = 0;
        this.rowCount = 0;
    }
    
    public void reset(final int newColCount, final int newRowCount) {
        this.clear();
        this.setColCount(newColCount);
        this.setRowCount(newRowCount);
    }
    
    public int getColCount() {
        return this.colCount;
    }
    
    public void setColCount(final int newColCount) {
        if (this.colCount == newColCount) {
            return;
        }
        final int count = (this.colCount <= newColCount) ? this.colCount : newColCount;
        for (int i = 0; i < this.rowCount; ++i) {
            final int[] oldRow = this.rowList.get(i);
            final int[] newRow = new int[newColCount];
            for (int j = 0; j < count; ++j) {
                newRow[j] = oldRow[j];
            }
            this.rowList.set(i, newRow);
        }
        this.colCount = newColCount;
    }
    
    public int getRowCount() {
        return this.rowCount;
    }
    
    public void setRowCount(final int newRowCount) {
        if (this.rowCount == newRowCount) {
            return;
        }
        for (int i = this.rowCount - 1; i >= newRowCount; --i) {
            this.rowList.remove(i);
        }
        for (int i = this.rowCount; i < newRowCount; ++i) {
            this.rowList.add(new int[this.colCount]);
        }
        this.rowCount = newRowCount;
    }
    
    public int get(final int x, final int y) {
        final int[] aRow = this.rowList.get(y);
        return aRow[x];
    }
    
    public void set(final int x, final int y, final int value) {
        final int[] aRow = this.rowList.get(y);
        aRow[x] = value;
    }
    
    public void insertCol(final int index) {
        for (int i = 0; i < this.rowCount; ++i) {
            final int[] oldRow = this.rowList.get(i);
            final int[] newRow = new int[this.colCount + 1];
            for (int j = 0; j < index; ++j) {
                newRow[j] = oldRow[j];
            }
            for (int j = index; j < this.colCount; ++j) {
                newRow[j + 1] = oldRow[j];
            }
        }
        ++this.colCount;
    }
    
    public void deleteCol(final int index) {
        for (int i = 0; i < this.rowCount; ++i) {
            final int[] oldRow = this.rowList.get(i);
            final int[] newRow = new int[this.colCount - 1];
            for (int j = 0; j < index; ++j) {
                newRow[j] = oldRow[j];
            }
            for (int j = index + 1; j < this.colCount; ++j) {
                newRow[j - 1] = oldRow[j];
            }
        }
        --this.colCount;
    }
    
    public void exchangeCol(final int index1, final int index2) {
        for (int i = 0; i < this.rowCount; ++i) {
            final int[] aRow = this.rowList.get(i);
            final int T = aRow[index1];
            aRow[index1] = aRow[index2];
            aRow[index2] = T;
        }
    }
    
    public void moveCol(final int index1, final int index2) {
        if (index1 == index2) {
            return;
        }
        for (int i = 0; i < this.rowCount; ++i) {
            final int[] aRow = this.rowList.get(i);
            final int T = aRow[index1];
            if (index1 > index2) {
                for (int j = index1 - 1; j >= index2; --j) {
                    aRow[j + 1] = aRow[j];
                }
            }
            else if (index1 < index2) {
                for (int j = index1 + 1; j < index2; ++j) {
                    aRow[j - 1] = aRow[j];
                }
            }
            aRow[index2] = T;
        }
    }
    
    public void insertRow(final int index) {
        this.rowList.add(index, new int[this.colCount]);
        ++this.rowCount;
    }
    
    public void deleteRow(final int index) {
        this.rowList.remove(index);
        --this.rowCount;
    }
    
    public void exchangeRow(final int index1, final int index2) {
        final int[] aRow = this.rowList.get(index1);
        this.rowList.set(index1, this.rowList.get(index2));
        this.rowList.set(index2, aRow);
    }
    
    public void moveRow(final int index1, final int index2) {
        final int[] aRow = this.rowList.get(index1);
        if (index1 > index2) {
            for (int i = index1 - 1; i >= index2; --i) {
                this.rowList.set(i + 1, this.rowList.get(i));
            }
        }
        else if (index1 < index2) {
            for (int i = index1 + 1; i < index2; ++i) {
                this.rowList.set(i - 1, this.rowList.get(i));
            }
        }
        this.rowList.set(index2, aRow);
    }
}
